/*-----------------------------------------------------------------*/
/*! 
  \file scscpinternal.h
  \brief SCSCP internal functions
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008, 2009,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr

   \bug M. GASTINEAU 20/04/09 : update for SCSCP 1.3
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#if !defined(__SCSCPINTERNAL_H__) 
#define __SCSCPINTERNAL_H__

#include <stdlib.h>

typedef struct SCSCP_optionsprocedurereturn* SCSCP_returnoptions;
typedef struct SCSCP_procedurecalloptions* SCSCP_calloptions;

//forward declaration
#ifndef LIBXML_DOTTED_VERSION
typedef struct _xmlDoc xmlDoc;
typedef xmlDoc* xmlDocPtr;
typedef struct _xmlNode* xmlNodePtr;
#endif

typedef struct _xmlNode* SCSCP_xmlnodeptr;
typedef struct _xmlAttr* SCSCP_xmlattrptr;


/*! debug level type */
typedef int SCSCP_debuglevel;



#if defined (__cplusplus)
extern "C" {
#endif /* (__cplusplus) */

/*! allocate memory of ncount bytes (similar to malloc) */
void* SCSCP_malloc(size_t ncount, SCSCP_status* status);
/*! duplicate a string (similar to strdup) */
char* SCSCP_strdup(const char*, SCSCP_status* status);

#if !HAVE_SNPRINTF
int SCSCP_snprintf(char *string, size_t n, const char *format, ...);
#define snprintf SCSCP_snprintf
#endif 

int SCSCP_initsocket();
/* macros pour compatibilite des sockets entre unix et windows */
#if !HAVE_WINDOWS_H || !defined(INVALID_SOCKET)
#define INVALID_SOCKET (-1) 
#define SOCKET_ERROR (-1) 
#define closesocket(sock) close(sock)
#endif



char* SCSCP_findPIstring(char *c, const char *versionname);


const char* SCSCP_xmlnode_findattr(SCSCP_xmlnodeptr curnode, const char *value);
int SCSCP_xmlnode_readOMIsizet(SCSCP_xmlnodeptr* curnode, size_t *value);
int SCSCP_xmlnode_readbeginOMOBJ(SCSCP_xmlnodeptr* curnode);
int SCSCP_xmlnode_readbeginOMA(SCSCP_xmlnodeptr* curnode);
int SCSCP_xmlnode_readbeginOMATTR(SCSCP_xmlnodeptr* curnode);
int SCSCP_xmlnode_readbeginOMATP(SCSCP_xmlnodeptr* curnode);
int SCSCP_xmlnode_readpairOMSOMIsizet(SCSCP_xmlnodeptr* curnode, const char *cdname, const char *symbolname, size_t *value);
int SCSCP_xmlnode_readpairOMSOMIint(SCSCP_xmlnodeptr* curnode, const char *cdname, const char *symbolname, int *value);
int SCSCP_xmlnode_readcheckOMS(SCSCP_xmlnodeptr* curnode, const char *cdname, const char *symbolname);
int SCSCP_xmlnode_movetotag(SCSCP_xmlnodeptr* curnode, const char *string, int explorechildren, int gotochildnode);

/*! set the error to value */
void SCSCP_status_seterror(SCSCP_status* status, int value);
/*! set the error to value  : EXECFAILED */
void SCSCP_status_setexecfailed(SCSCP_status* status, const char *cdname, const char *symbolname, const char *msg);



/*-----------------------------------------------------------------*/
//! buffer for len for scscp Processing instruction
/*-----------------------------------------------------------------*/
#define SCSCP_PI_MAXLENBUFFER 4096


#if defined (__cplusplus)
}
#endif /* (__cplusplus) */

#endif /*__SCSCPINTERNAL_H__*/
