/*-----------------------------------------------------------------*/
/*! 
  \file scscptypes.h
  \brief SCSCP basic types 
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008,  M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr

  @bug M. GASTINEAU 12/09/08 : update for SCSCP 1.2
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
This file is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute this file under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.

*/
/*-----------------------------------------------------------------*/


#ifndef __SCSCPTYPES_H__
#define __SCSCPTYPES_H__


/*-----------------------------------------------------------------*/
/* types */
/*-----------------------------------------------------------------*/

//! string type
typedef char* SCSCP_string;
   
/*! const string type */
typedef const char* const_SCSCP_string;

/*! time limits type */
typedef size_t SCSCP_time;

/*! memory limits type */
typedef size_t SCSCP_memory;


/*! status type */
/*typedef int SCSCP_status;*/
typedef struct
{
 int   m_code;        /*!< code value */
 /*extended attributes  : only for PROCEDURETERMINATED */
 char *m_cdname;      /*!< name of the content dictionnary */
 char *m_symbolname;  /*!< name of the symbol */
 char *m_message;     /*!< message content */
 char *m_fullformattedmessage; /*! full formatted message for strerror */
} SCSCP_status;

/*-----------------------------------------------------------------*/
/*! enumeration of return type */
/*-----------------------------------------------------------------*/
typedef enum
{
 SCSCP_option_return_object, /*!< return the object*/
 SCSCP_option_return_cookie, /*!< return a cookie*/
 SCSCP_option_return_nothing /*!< return nothing*/
} SCSCP_option_return;

/*-----------------------------------------------------------------*/
/*! enumeration of message type*/
/*-----------------------------------------------------------------*/
typedef enum
{
 SCSCP_msgtype_ProcedureTerminated, /*!< message received is procedure terminated */
 SCSCP_msgtype_ProcedureCompleted,  /*!< message received is procedure completed */
 SCSCP_msgtype_ProcedureCall,       /*!< message received is procedure call */
 SCSCP_msgtype_Interrupt            /*!< message received is interrupt */
} SCSCP_msgtype;
 



#endif /*__SCSCPTYPES_H__*/
