/*-----------------------------------------------------------------*/
/*! 
  \file scscpsscommon.c
  \brief common subroutine for server tests
  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2008, 2009, M. Gastineau, IMCCE-CNRS
   email of the author : gastineau@imcce.fr
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
 This library is governed by the CeCILL-C license under French law and
abiding by the rules of distribution of free software.  You can  use, 
modify and/ or redistribute the software under the terms of the CeCILL-C
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C license and that you accept its terms.
*/
/*-----------------------------------------------------------------*/

#if HAVE_CONFIG_H
#include "scscpconfig.h"
#endif /*HAVE_CONFIG_H*/
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#if HAVE_SYS_WAIT_H
#include <sys/wait.h>
#if HAVE_UNISTD_H
#include <unistd.h>
#endif
#endif
#include "scscp.h"
#include "scscpsscommon.h"


#if HAVE_FORK
typedef pid_t ospid_t; 
#elif HAVE_WIN32API
typedef PROCESS_INFORMATION ospid_t;
#endif

/*  id of the server */
static ospid_t pserverid;

/*-----------------------------------------------------------------*/
/*! start a new thread which will run the server. update pserverid */
/*-----------------------------------------------------------------*/
int startbackgroundserver(int typeanswer)
{
 int res = 0;
 char bufanswer[10];
 int sleeptime = 3;
 
 sprintf(bufanswer, "%d",typeanswer);

#if HAVE_FORK
   pserverid = fork();
   switch(pserverid)
   {
   case -1:
      res = 0;
      break;
      
   case 0: /* child */ 
       printf("child process\n");
      if (execl("scscpservertest", "scscpservertest", bufanswer, NULL)==-1)
      {
       res = 0;
       printf("startbackgroundserver failed on exec\n");
      }
      break;
      
   default: /* parent */ 
       res = 1;
       printf("parent process : child pid= %d\n",(int)pserverid);
       break;
   }

#elif HAVE_WIN32API

   {
   STARTUPINFO siStartInfo;
   BOOL bFuncRetn = FALSE; 
   char cmd[1024];
   
   sprintf(cmd,"scscpservertest.exe %d", typeanswer);
   
   ZeroMemory( &pserverid, sizeof(PROCESS_INFORMATION) );
   ZeroMemory( &siStartInfo, sizeof(STARTUPINFO) );
   siStartInfo.cb = sizeof(STARTUPINFO); 
    
    res = CreateProcess(NULL, 
      cmd,      /* command line */
      NULL,          /* process security attributes */
      NULL,          /* primary thread security attributes */
      FALSE,          /* handles are inherited */
      CREATE_NEW_PROCESS_GROUP, /* creation flags */
      NULL,          /* use parent's environment */
      NULL,          /* use parent's current directory */
      &siStartInfo,  /* STARTUPINFO pointer */
      &pserverid);  /* receives PROCESS_INFORMATION */

  if (res==0)
   {
     printf("res=%d\n",res);
     printf( "CreateProcess failed (%d)\n", GetLastError() );
   }
  }
  
#endif /*HAVE_WIN32API*/
 printf("client sleep %d seconds to start background server\n", sleeptime);
 mysleep(sleeptime);
 
 return res;
}

/*-----------------------------------------------------------------*/
/*! send the stop action to the thread of the server  
   @bug M. GASTINEAU 13/01/2009 : send CTRL+BREAK 
   to stop the server on windows
*/
/*-----------------------------------------------------------------*/
int stopbackgroundserver()
{
 int res = 0;
 int status;
 
#if HAVE_FORK
 printf("send kill to  pid= %d\n",(int)pserverid);
 kill(pserverid, SIGUSR2);
 waitpid(pserverid, &status, WUNTRACED);
 
 if (WIFEXITED(status)) 
 {
   res = (WEXITSTATUS(status) != 0);
 }
#elif HAVE_WIN32API
 GenerateConsoleCtrlEvent(CTRL_BREAK_EVENT, pserverid.dwProcessId);
 WaitForSingleObject( pserverid.hProcess, INFINITE );
 CloseHandle(pserverid.hProcess);
 CloseHandle(pserverid.hThread);
#endif 
 return res;
}

/*-----------------------------------------------------------------*/
/*! sleep for some seconds
   @param sec (in) number of seconds
*/
/*-----------------------------------------------------------------*/
void mysleep(int sec)
{
#if HAVE_FORK
 sleep(sec);
#elif HAVE_WIN32API
 Sleep(sec*1000);
#endif 
}

